;---------------------------------------------------------------------
; file          : readline.inc
; created       : 2000/08/20
; description   : read a line with editting
;
;         Copyright (C) 2000 Jun Mizutani <mizutani.jun@nifty.ne.jp>
;
; enter    eax : BufferSize
;          ebx : Buffer Address
; return   eax : no. of char
;---------------------------------------------------------------------

%ifndef __READLINE_INC
%define __READLINE_INC

%include "stdio.inc"
%include "termios.inc"

READ_LINE:
                push   esi
                push   edi
                push   ebx
                push   ecx
                push   edx
                mov    edi, ebx          ; Input Buffer
                mov    edx, eax          ; BufferSize
                xor    ecx, ecx
                mov    esi, ecx          ; current position
    .next_char:
                call   InChar
                cmp    al, 0x1B          ; ESC ?
                jnz    .bs
                call   translate_key_seq

    .bs:        cmp    al, 0x08          ; BS ?
                jnz    .delete
                test   esi, esi          ; if cp = 0 then next_char
                jz     .next_char
                dec    esi
                mov    ebx, CURSOR_LEFT
                call   OutPString
                jmp    short .del1

    .delete:
                cmp    al, 0x04          ; ^D ?
                jnz    .cursor_left
    .del1
                cmp    esi, ecx          ; if cp < eol then del2
                jb     .del2
                jmp    short .next_char
    .del2
                dec    ecx
                push   esi               ; p = cp
    .del3       cmp    esi, ecx          ; while(p<eol){buf[p]=buf[p+1]; p++}
                je     .del4
                mov    al, [edi+esi+1]
                mov    [edi + esi], al
                inc    esi
                jmp    short .del3
    .del4
                pop    esi
                mov    ebx, DEL_AT_CURSOR
                call   OutPString
                jmp    short .next_char

    .cursor_left:
                cmp    al, 0x02          ; ^B
                jnz    .cursor_right
                test   esi, esi          ; if cp = 0 then next_char
                jz     .next_char
                dec    esi
                mov    ebx, CURSOR_LEFT
                call   OutPString
                jmp    short .next_char

    .cursor_right:
                cmp    al, 0x06          ; ^F
                jnz    .enter_key
                cmp    ecx, esi          ; if cp = eol then next_char
                je     .next_char
                inc    esi
                mov    ebx, CURSOR_RIGHT
                call   OutPString
                jmp    .next_char

    .enter_key:
                cmp    al, 0x0A          ; enter ?
                jz     .in_exit
                cmp    al, 0x20
                jb     near .next_char   ; illegal chars

    .in_printable:
                inc    ecx
                inc    esi
                cmp    ecx, edx          ; buffer size
                jae    .in_toolong
                cmp    esi, ecx
                jb     .insert
                call   OutChar
                mov    [edi + esi-1], al
                jmp    .next_char
    .insert
                call   OutChar
                push   eax
                push   ecx               ; p = eol
                dec    ecx
    .ins1       cmp    esi, ecx          ; while(p=>cp){buf[p]=buf[p-1]; p--}
                ja     .ins2
                mov    al, [edi + ecx - 1]
                mov    [edi + ecx], al
                dec    ecx
                jmp    short .ins1
    .ins2
                pop    ecx
                pop    eax
                mov    [edi + esi - 1] ,al
                call   print_line_after_cp
                jmp    .next_char

    .in_toolong:
                dec    ecx
                dec    esi
                jmp    .next_char

    .in_exit:
                mov    byte [edi + ecx], 0
                call   NewLine
                mov    eax, ecx
                pop    edx
                pop    ecx
                pop    ebx
                pop    edi
                pop    esi
                ret

    print_line_after_cp:
                push   eax
                push   edx
                mov    ebx, SAVE_CURSOR
                call   OutPString
                mov    ebx, CLEAR_EOL
                call   OutPString
                lea    eax, [esi+edi]
                mov    edx, ecx
                sub    edx, esi
                call   OutString
                mov    ebx, RESTORE_CURSOR
                call   OutPString
                pop    edx
                pop    eax
                ret

    translate_key_seq:
                call   InChar
                cmp    al, '['
                jnz    .exit
                call   InChar
    .tk0:       cmp    al, 'A'
                jnz    .tk1
                mov    al, 'P' - 0x40  ; ^P
                ret
    .tk1:       cmp    al, 'B'
                jnz    .tk2
                mov    al, 'N' - 0x40  ; ^N
                ret
    .tk2:       cmp    al, 'C'
                jnz    .tk3
                mov    al, 'F' - 0x40  ; ^F
                ret
    .tk3:       cmp    al, 'D'
                jnz    .exit
                mov    al, 'B' - 0x40  ; ^B
                ret
    .exit       mov    al, 0
                ret

SAVE_CURSOR     db     2, 0x1B, '7'             ; ^[7
RESTORE_CURSOR  db     2, 0x1B, '8'             ; ^[8
DEL_AT_CURSOR   db     4, 0x1B, '[', 'P', 1     ; ^[[P1
CURSOR_RIGHT    db     4, 0x1B, '[', 'C', 1     ; ^[[C1
CURSOR_LEFT     db     4, 0x1B, '[', 'D', 1     ; ^[[D1
CLEAR_EOL       db     4, 0x1B, '[', 'K', 0     ; ^[[K0

%endif
