;---------------------------------------------------------------------
; file          : termios.inc
; created       : 2000/08/19
; assembler     : nasm 0.98
; description   : termios structures and constants
; derived from  : linux-2.4.0-test6/include/asm-i386/termbits.h,
;               : termios.h, ioctl.h, ioctls.h
;
;Copyright (C) 2000 Jun Mizutani <mizutani.jun@nifty.ne.jp>
;---------------------------------------------------------------------

%ifndef __TERMIOS_INC
%define __TERMIOS_INC

%include "vartype.inc"

;---------------------------------------------------------------------
; /usr/src/linux/include/asm-i386/termbits.h

%assign NCCS 19

struc termios
    .c_iflag    UINT   1       ; input mode flags
    .c_oflag    UINT   1       ; output mode flags
    .c_cflag    UINT   1       ; control mode flags
    .c_lflag    UINT   1       ; local mode flags
    .c_line     UCHAR  1       ; line discipline
    .c_cc       UCHAR  NCCS    ; control characters
endstruc

;  c_cc characters
%assign VINTR       0
%assign VQUIT       1
%assign VERASE      2
%assign VKILL       3
%assign VEOF        4
%assign VTIME       5
%assign VMIN        6
%assign VSWTC       7
%assign VSTART      8
%assign VSTOP       9
%assign VSUSP       10
%assign VEOL        11
%assign VREPRINT    12
%assign VDISCARD    13
%assign VWERASE     14
%assign VLNEXT      15
%assign VEOL2       16

;  c_iflag bits
%assign IGNBRK  0000001q
%assign BRKINT  0000002q
%assign IGNPAR  0000004q
%assign PARMRK  0000010q
%assign INPCK   0000020q
%assign ISTRIP  0000040q
%assign INLCR   0000100q
%assign IGNCR   0000200q
%assign ICRNL   0000400q
%assign IUCLC   0001000q
%assign IXON    0002000q
%assign IXANY   0004000q
%assign IXOFF   0010000q
%assign IMAXBEL 0020000q

;  c_oflag bits
%assign OPOST   0000001q
%assign OLCUC   0000002q
%assign ONLCR   0000004q
%assign OCRNL   0000010q
%assign ONOCR   0000020q
%assign ONLRET  0000040q
%assign OFILL   0000100q
%assign OFDEL   0000200q
%assign NLDLY   0000400q
%assign NL0     0000000q
%assign NL1     0000400q
%assign CRDLY   0003000q
%assign CR0     0000000q
%assign CR1     0001000q
%assign CR2     0002000q
%assign CR3     0003000q
%assign TABDLY  0014000q
%assign TAB0    0000000q
%assign TAB1    0004000q
%assign TAB2    0010000q
%assign TAB3    0014000q
%assign XTABS   0014000q
%assign BSDLY   0020000q
%assign BS0     0000000q
%assign BS1     0020000q
%assign VTDLY   0040000q
%assign VT0     0000000q
%assign VT1     0040000q
%assign FFDLY   0100000q
%assign FF0     0000000q
%assign FF1     0100000q

;  c_cflag bit meaning
%assign CBAUD   0010017q
%assign B0      0000000q         ;  hang up
%assign B50     0000001q
%assign B75     0000002q
%assign B110    0000003q
%assign B134    0000004q
%assign B150    0000005q
%assign B200    0000006q
%assign B300    0000007q
%assign B600    0000010q
%assign B1200   0000011q
%assign B1800   0000012q
%assign B2400   0000013q
%assign B4800   0000014q
%assign B9600   0000015q
%assign B19200  0000016q
%assign B38400  0000017q
%assign EXTA    B19200
%assign EXTB    B38400
%assign CSIZE   0000060q
%assign CS5     0000000q
%assign CS6     0000020q
%assign CS7     0000040q
%assign CS8     0000060q
%assign CSTOPB  0000100q
%assign CREAD   0000200q
%assign PARENB  0000400q
%assign PARODD  0001000q
%assign HUPCL   0002000q
%assign CLOCAL  0004000q
%assign CBAUDEX 0010000q
%assign  B57600 0010001q
%assign B115200 0010002q
%assign B230400 0010003q
%assign B460800 0010004q
%assign B500000 0010005q
%assign B576000 0010006q
%assign B921600 0010007q
%assign B1000000 0010010q
%assign B1152000 0010011q
%assign B1500000 0010012q
%assign B2000000 0010013q
%assign B2500000 0010014q
%assign B3000000 0010015q
%assign B3500000 0010016q
%assign B4000000 0010017
%assign CIBAUD  002003600000q  ;  input baud rate (not used)
%assign CMSPAR  010000000000q  ;  mark or space (stick) parity
%assign CRTSCTS 020000000000q  ;  flow control

;  c_lflag bits
%assign ISIG    0000001q
%assign ICANON  0000002q
%assign XCASE   0000004q
%assign ECHO    0000010q
%assign ECHOE   0000020q
%assign ECHOK   0000040q
%assign ECHONL  0000100q
%assign NOFLSH  0000200q
%assign TOSTOP  0000400q
%assign ECHOCTL 0001000q
%assign ECHOPRT 0002000q
%assign ECHOKE  0004000q
%assign FLUSHO  0010000q
%assign PENDIN  0040000q
%assign IEXTEN  0100000q

;  tcflow() and TCXONC use these
%assign TCOOFF          0
%assign TCOON           1
%assign TCIOFF          2
%assign TCION           3

;  tcflush() and TCFLSH use these
%assign TCIFLUSH        0
%assign TCOFLUSH        1
%assign TCIOFLUSH       2

;  tcsetattr uses these
%assign TCSANOW         0
%assign TCSADRAIN       1
%assign TCSAFLUSH       2

;---------------------------------------------------------------------
; /usr/src/linux/include/asm-i386/termios.h

struc winsize
        .ws_row     USHORT     1
        .ws_col     USHORT     1
        .ws_xpixel  USHORT     1
        .ws_ypixel  USHORT     1
endstruc

%assign NCC 8

struc termio
        .c_iflag    USHORT     1       ; input mode flags
        .c_oflag    USHORT     1       ; output mode flags
        .c_cflag    USHORT     1       ; control mode flags
        .c_lflag    USHORT     1       ; local mode flags
        .c_line     UCHAR      1       ; line discipline
        .c_cc       UCHAR      NCC     ; control characters
endstruc

;  modem lines
%assign TIOCM_LE        0x001
%assign TIOCM_DTR       0x002
%assign TIOCM_RTS       0x004
%assign TIOCM_ST        0x008
%assign TIOCM_SR        0x010
%assign TIOCM_CTS       0x020
%assign TIOCM_CAR       0x040
%assign TIOCM_RNG       0x080
%assign TIOCM_DSR       0x100
%assign TIOCM_CD        TIOCM_CAR
%assign TIOCM_RI        TIOCM_RNG
%assign TIOCM_OUT1      0x2000
%assign TIOCM_OUT2      0x4000
%assign TIOCM_LOOP      0x8000

;  ioctl (fd, TIOCSERGETLSR, &result) where result may be as below

;  line disciplines
%assign N_TTY           0
%assign N_SLIP          1
%assign N_MOUSE         2
%assign N_PPP           3
%assign N_STRIP         4
%assign N_AX25          5
%assign N_X25           6   ; X.25 async
%assign N_6PACK         7
%assign N_MASC          8   ; Reserved for Mobitex module <kaz@cafe.net>
%assign N_R3964         9   ; Reserved for Simatic R3964 module
%assign N_PROFIBUS_FDL  10  ; Reserved for Profibus <Dave@mvhi.com>
; Linux IR - http://www.cs.uit.no/~dagb/irda/irda.html
%assign N_IRDA          11
; SMS block mode - for talking to GSM data cards about SMS messages
%assign N_SMSBLOCK      12
%assign N_HDLC          13  ;  synchronous HDLC
%assign N_SYNC_PPP      14  ;  synchronous PPP

;---------------------------------------------------------------------
;      intr=^C         quit=^\         erase=del       kill=^U
;      eof=^D          vtime=\0        vmin=\1         sxtc=\0
;      start=^Q        stop=^S         susp=^Z         eol=\0
;      reprint=^R      discard=^U      werase=^W       lnext=^V
;      eol2=\0

INIT_C_CC db "\003\034\177\025\004\0\1\0\021\023\032\0\022\017\027\026\0"

;---------------------------------------------------------------------
; /usr/src/linux/include/asm-i386/ioctl.h

%assign _IOC_NRBITS     8
%assign _IOC_TYPEBITS   8
%assign _IOC_SIZEBITS   14
%assign _IOC_DIRBITS    2

%assign _IOC_NRMASK     ((1 << _IOC_NRBITS)-1)
%assign _IOC_TYPEMASK   ((1 << _IOC_TYPEBITS)-1)
%assign _IOC_SIZEMASK   ((1 << _IOC_SIZEBITS)-1)
%assign _IOC_DIRMASK    ((1 << _IOC_DIRBITS)-1)

%assign _IOC_NRSHIFT    0
%assign _IOC_TYPESHIFT  (_IOC_NRSHIFT+_IOC_NRBITS)
%assign _IOC_SIZESHIFT  (_IOC_TYPESHIFT+_IOC_TYPEBITS)
%assign _IOC_DIRSHIFT   (_IOC_SIZESHIFT+_IOC_SIZEBITS)

; Direction bits.

%assign _IOC_NONE       0
%assign _IOC_WRITE      1
%assign _IOC_READ       2

%define _IOC(dir,type,nr,size)  (((dir)<< _IOC_DIRSHIFT)|((type)<<_IOC_TYPESHIFT)|((nr) << _IOC_NRSHIFT)|((size) << _IOC_SIZESHIFT))

;  used to create numbers
%define _IO(type,nr)            _IOC(_IOC_NONE,(type),(nr),0)
%define _IOR(type,nr,size)      _IOC(_IOC_READ,(type),(nr),(size))
%define _IOW(type,nr,size)      _IOC(_IOC_WRITE,(type),(nr),(size))
%define _IOWR(type,nr,size) _IOC(_IOC_READ|_IOC_WRITE,(type),(nr),(size))

;  used to decode ioctl numbers..
%define _IOC_DIR(nr)            (((nr) >> _IOC_DIRSHIFT) & _IOC_DIRMASK)
%define _IOC_TYPE(nr)           (((nr) >> _IOC_TYPESHIFT) & _IOC_TYPEMASK)
%define _IOC_NR(nr)             (((nr) >> _IOC_NRSHIFT) & _IOC_NRMASK)
%define _IOC_SIZE(nr)           (((nr) >> _IOC_SIZESHIFT) & _IOC_SIZEMASK)

;  ...and for the drivers/sound files...

%assign IOC_IN          (_IOC_WRITE << _IOC_DIRSHIFT)
%assign IOC_OUT         (_IOC_READ << _IOC_DIRSHIFT)
%assign IOC_INOUT       ((_IOC_WRITE|_IOC_READ) << _IOC_DIRSHIFT)
%assign IOCSIZE_MASK    (_IOC_SIZEMASK << _IOC_SIZESHIFT)
%assign IOCSIZE_SHIFT   (_IOC_SIZESHIFT)

;---------------------------------------------------------------------
; /usr/src/linux/include/asm-i386/ioctls.h
;  0x54 is just a magic number to make these relatively unique ('T')

%assign TCGETS          0x5401
%assign TCSETS          0x5402
%assign TCSETSW         0x5403
%assign TCSETSF         0x5404
%assign TCGETA          0x5405
%assign TCSETA          0x5406
%assign TCSETAW         0x5407
%assign TCSETAF         0x5408
%assign TCSBRK          0x5409
%assign TCXONC          0x540A
%assign TCFLSH          0x540B
%assign TIOCEXCL        0x540C
%assign TIOCNXCL        0x540D
%assign TIOCSCTTY       0x540E
%assign TIOCGPGRP       0x540F
%assign TIOCSPGRP       0x5410
%assign TIOCOUTQ        0x5411
%assign TIOCSTI         0x5412
%assign TIOCGWINSZ      0x5413
%assign TIOCSWINSZ      0x5414
%assign TIOCMGET        0x5415
%assign TIOCMBIS        0x5416
%assign TIOCMBIC        0x5417
%assign TIOCMSET        0x5418
%assign TIOCGSOFTCAR    0x5419
%assign TIOCSSOFTCAR    0x541A
%assign FIONREAD        0x541B
%assign TIOCINQ         FIONREAD
%assign TIOCLINUX       0x541C
%assign TIOCCONS        0x541D
%assign TIOCGSERIAL     0x541E
%assign TIOCSSERIAL     0x541F
%assign TIOCPKT         0x5420
%assign FIONBIO         0x5421
%assign TIOCNOTTY       0x5422
%assign TIOCSETD        0x5423
%assign TIOCGETD        0x5424
%assign TCSBRKP         0x5425  ;  Needed for POSIX tcsendbreak()
%assign TIOCTTYGSTRUCT  0x5426  ;  For debugging only
%assign TIOCSBRK        0x5427  ;  BSD compatibility
%assign TIOCCBRK        0x5428  ;  BSD compatibility
%assign TIOCGSID        0x5429  ;  Return the session ID of FD
;  Get Pty Number (of pty-mux device)
%assign TIOCGPTN        _IOR('T',0x30, 4)
;  Lock/unlock Pty
%assign TIOCSPTLCK      _IOW('T',0x31, 4)

%assign FIONCLEX        0x5450  ;  these numbers need to be adjusted.
%assign FIOCLEX         0x5451
%assign FIOASYNC        0x5452
%assign TIOCSERCONFIG   0x5453
%assign TIOCSERGWILD    0x5454
%assign TIOCSERSWILD    0x5455
%assign TIOCGLCKTRMIOS  0x5456
%assign TIOCSLCKTRMIOS  0x5457
%assign TIOCSERGSTRUCT  0x5458  ;  For debugging only
%assign TIOCSERGETLSR   0x5459  ;  Get line status register
%assign TIOCSERGETMULTI 0x545A  ;  Get multiport config
%assign TIOCSERSETMULTI 0x545B  ;  Set multiport config

%assign TIOCMIWAIT      0x545C  ;  wait for a change on serial input line(s)
%assign TIOCGICOUNT     0x545D  ;  read serial port inline interrupt counts
%assign TIOCGHAYESESP   0x545E  ;  Get Hayes ESP configuration
%assign TIOCSHAYESESP   0x545F  ;  Set Hayes ESP configuration

;  Used for packet mode
%assign TIOCPKT_DATA             0
%assign TIOCPKT_FLUSHREAD        1
%assign TIOCPKT_FLUSHWRITE       2
%assign TIOCPKT_STOP             4
%assign TIOCPKT_START            8
%assign TIOCPKT_NOSTOP          16
%assign TIOCPKT_DOSTOP          32

%assign TIOCSER_TEMT    0x01    ;  Transmitter physically empty

%endif

